# -*- coding: utf-8 -*-

def to_string(v):
    version_string = '%d.%d.%d' % v[0:3]
    try:
        version_string += '%s' % v[3]
        version_string += '%d' % v[4]
    except IndexError:
        pass
    return version_string



# -*- coding: utf-8 -*-

# (major, minor, bugfix): (1, 0, 0) -> 1.0.0
# (major, minor, bugfix, phase): (1, 1, 5, 'alpha') -> 1.0.5alpha
# (major, minor, bugfix, phase, phaseversion): (1, 1, 0, 'rc', 5) -> 1.1.0rc5

VERSION = (1, 0, 1)


def get_version():
    from . import version
    return version.to_string(VERSION)



from __future__ import unicode_literals

from email.header import Header

import django
from django.http import HttpResponse
from django.views.generic.list import MultipleObjectMixin, BaseListView
from django.core.exceptions import ImproperlyConfigured

try:
    import unicodecsv as csv
except ImportError:  # unicodecsv is unnecessary on Python 3
    import csv

from separated.utils import Getter


def encode_header(value):
    return Header(value, 'utf-8').encode()


class CsvResponse(HttpResponse):
    def __init__(self, filename, content_type='text/csv', **kwargs):
        super(CsvResponse, self).__init__(content_type=content_type, **kwargs)
        disposition = 'attachment; filename="{0}"'.format(filename)
        # BBB: Django 1.4 and earlier didn't support non-ASCII headers.  Later
        # versions do this for us.
        if django.VERSION < (1, 5):
            disposition = encode_header(disposition)
        self['Content-Disposition'] = disposition


class CsvResponseMixin(MultipleObjectMixin):
    """
    A ListView mixin that returns a CsvResponse.
    """
    response_class = CsvResponse
    columns = None
    output_headers = True
    filename = '{model_name}_list.csv'

    def render_to_response(self, context, **kwargs):
        queryset = context['object_list']
        model = queryset.model
        response = self.response_class(
            filename=self.get_filename(model),
        )

        writer = csv.writer(response)

        if self.output_headers:
            writer.writerow(self.get_header_row(model))

        for obj in queryset:
            writer.writerow(self.get_row(obj))

        return response

    def format_header(self, column):
        if self.output_headers:
            try:
                return column.short_description
            except AttributeError:
                raise ImproperlyConfigured(
                    "If you pass a function as an accessor,"
                    " please provide a column title."
                )

    def get_header_row(self, model):
        return [c[1] for c in self.get_normalized_columns(model)]

    def get_row(self, obj):
        return [c[0](obj)
                for c in self.get_normalized_columns(type(obj))]

    def get_normalized_columns(self, model):
        return map(self._normalize_column, self.get_columns(model))

    def get_columns(self, model):
        if self.columns is None:
            raise ImproperlyConfigured('Please set the columns.')
        return self.columns

    def get_filename(self, model):
        opts = model._meta
        model_name = getattr(opts, 'model_name', opts.module_name)
        return self.filename.format(
            model_name=model_name,
        )

    def _normalize_column(self, column):
        # column can either be a 2-tuple of (accessor, header), or just an
        # accessor.  accessor will be passed to Getter, and we will get the
        # header off of the Getter. Returns a 2-tuple of (Getter, header).
        if isinstance(column, (tuple, list)):
            column = self._normalize_getter(column[0]), column[1]
        else:
            column = self._normalize_getter(column)
            column = (column, self.format_header(column))
        return column

    _getter_cache = {}

    def _normalize_getter(self, getter):
        if not getter in self._getter_cache:
            self._getter_cache[getter] = Getter(getter)
        return self._getter_cache[getter]


class CsvView(CsvResponseMixin, BaseListView):
    """
    A ListView that returns a CsvResponse.
    """



from operator import attrgetter
from functools import partial


def get_pretty_name(accessor):
    return accessor.replace('_', ' ') \
        .replace('.', ' ') \
        .capitalize()


def Getter(accessor, normalizer=lambda x: x):
    """
    Returns a function that will access an attribute off of an object.  If that
    attribute is callable, it will call it.  Accepts a normalizer to call on
    the value at the end.
    """
    if not callable(accessor):
        short_description = get_pretty_name(accessor)
        accessor = attrgetter(accessor)
    else:
        short_description = getattr(accessor, 'short_description', None)

    def getter(obj):
        ret = accessor(obj)
        # handle things like get_absolute_url
        if callable(ret):
            ret = ret()
        return normalizer(ret)

    if short_description:
        getter.short_description = short_description

    return getter


# Should these be i18nized?
bool2string_map = {True: 'Yes', False: 'No'}

BooleanGetter = partial(Getter, normalizer=bool2string_map.get)


def DisplayGetter(accessor, *args, **kwargs):
    """
    Returns a Getter that gets the display name for a model field with choices.
    """
    short_description = get_pretty_name(accessor)
    accessor = 'get_%s_display' % accessor
    getter = Getter(accessor, *args, **kwargs)
    getter.short_description = short_description
    return getter



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse
from django.core.exceptions import ImproperlyConfigured

from separated.views import CsvView, encode_header
from separated.utils import Getter, BooleanGetter

from testproject.testproject.models import Car, Manufacturer
from testproject.testproject.admin import (
    OverrideExportColumnsAdmin, OverrideExportViewAdmin,
    NoColumnsExportAdmin, ExportColumnsAndExportViewAdmin,
)


class StringAccessorTest(TestCase):
    def setUp(self):
        self.manufacturer = Manufacturer.objects.create(
            name='Jeep',
        )
        self.car = self.manufacturer.car_set.create(
            name='Grand Cherokee',
        )

    def test_lambda(self):
        class View(CsvView):
            model = Car
            columns = [lambda x: x.name]
            output_headers = False

        self.assertEqual(View().get_row(self.car), ['Grand Cherokee'])

    def test_simple_attr(self):
        class View(CsvView):
            model = Car
            columns = ['name']

        self.assertEqual(View().get_row(self.car), ['Grand Cherokee'])

    def test_dotted_path(self):
        class View(CsvView):
            model = Car
            columns = ['manufacturer.name']

        self.assertEqual(View().get_row(self.car), ['Jeep'])

    def test_callable(self):
        class View(CsvView):
            model = Car
            columns = ['get_display_name']

        self.assertEqual(View().get_row(self.car), ['GRAND CHEROKEE'])


class GetterTest(TestCase):
    def setUp(self):
        self.manufacturer = Manufacturer.objects.create(
            name='Jeep',
        )
        self.car = self.manufacturer.car_set.create(
            name='Grand Cherokee',
        )

    def test_lambda(self):
        get = Getter(lambda x: x.name)
        self.assertEqual(get(self.car), 'Grand Cherokee')

    def test_simpleattr(self):
        get = Getter('name')
        self.assertEqual(get(self.car), 'Grand Cherokee')
        self.assertEqual(get.short_description, 'Name')

    def test_dotted_path(self):
        get = Getter('manufacturer.name')
        self.assertEqual(get(self.car), 'Jeep')

    def test_callable(self):
        get = Getter('get_display_name')
        self.assertEqual(get(self.car), 'GRAND CHEROKEE')

    def test_normalizer(self):
        get = Getter('name', normalizer=lambda x: x.upper())
        self.assertEqual(get(self.car), 'GRAND CHEROKEE')

    def test_boolean_getter(self):
        get = BooleanGetter('is_admin')
        # Administrator car??
        self.car.is_admin = True
        self.assertEqual(get(self.car), 'Yes')
        self.car.is_admin = False
        self.assertEqual(get(self.car), 'No')

    def test_nested_getters(self):
        get = BooleanGetter('is_admin')
        get = Getter(get)
        self.car.is_admin = True
        self.assertEqual(get(self.car), 'Yes')
        self.assertEqual(get.short_description, 'Is admin')


class ColumnNormalizerTest(TestCase):
    def test_generate_header(self):
        def getter(obj):
            return obj.thing
        getter.short_description = 'A Getter'

        class View(CsvView):
            columns = ['name', 'manufacturer.name', 'display_name', getter]

        self.assertEqual(View().get_header_row(Manufacturer), [
            'Name',
            'Manufacturer name',
            'Display name',
            'A Getter',
        ])

    def test_raises_exception_when_column_header_cannot_be_generated(self):
        class View(CsvView):
            columns = [lambda x: x.name]

        self.assertRaises(ImproperlyConfigured, View().get_header_row, Manufacturer)

        class SuppressView(View):
            output_headers = False

        # Doesn't raise
        SuppressView().get_header_row(Manufacturer)

    def test_can_supply_headers(self):
        class View(CsvView):
            columns = [
                'name',
                ('manufacturer.name', 'Thing'),
                'display_name',
                ('get_absolute_url', 'Description'),
            ]

        self.assertEqual(View().get_header_row(Manufacturer), [
            'Name',
            'Thing',
            'Display name',
            'Description',
        ])


class CsvViewTest(TestCase):
    def setUp(self):
        self.manufacturer = Manufacturer.objects.create(
            name='你好凯兰',
        )

    def test_unicode_csv_output(self):
        # In Python 2, the built-in csv library doesn't handle unicode.
        response = self.client.get(reverse('manufacturers'))
        expected = "Name,Number of models\r\n你好凯兰,0\r\n".encode('utf8')
        self.assertEqual(response.content, expected)

    def test_content_type(self):
        response = self.client.get(reverse('manufacturers'))
        self.assertEqual(response['Content-Type'], 'text/csv')

    def test_filename_autogenerated(self):
        response = self.client.get(reverse('manufacturers'))
        expected = 'attachment; filename="manufacturer_list.csv"'
        self.assertEqual(response['Content-Disposition'], expected)

    def test_unicode_filename(self):
        response = self.client.get(reverse('unicode_filename'))
        # It is kinda weird to have Unicode in the headers.  Copied the encoder
        # from Django.
        expected = encode_header('attachment; filename="áèïôų.csv"')
        self.assertEqual(response['Content-Disposition'], expected)


class CsvExportAdminTest(TestCase):
    def setUp(self):
        Manufacturer.objects.create(
            name='Manufacturer A',
        )
        Manufacturer.objects.create(
            name='Manufacturer B',
        )
        self.factory = RequestFactory()

    def test_csv_export_columns(self):
        admin = OverrideExportColumnsAdmin(Manufacturer, 'testproject')
        # actions are only ever POSTed to.
        request = self.factory.post('/')
        queryset = Manufacturer.objects.all()
        response = admin.export_csv_action(request, queryset)
        self.assertEqual(response.status_code, 200)
        expected = "Name,Number of models\r\nManufacturer A,0\r\nManufacturer B,0\r\n".encode('utf8')
        self.assertEqual(response.content, expected)

    def test_csv_export_view_class(self):
        admin = OverrideExportViewAdmin(Manufacturer, 'testproject')
        request = self.factory.post('/')
        queryset = Manufacturer.objects.all()
        response = admin.export_csv_action(request, queryset)
        expected = "0,Manufacturer A\r\n0,Manufacturer B\r\n".encode('utf8')
        self.assertEqual(response.content, expected)

    def test_csv_export_columns_overrides_views_columns(self):
        admin = ExportColumnsAndExportViewAdmin(Manufacturer, 'testproject')
        request = self.factory.post('/')
        queryset = Manufacturer.objects.all()
        response = admin.export_csv_action(request, queryset)
        expected = "Manufacturer A,0\r\nManufacturer B,0\r\n".encode('utf8')
        self.assertEqual(response.content, expected)

    def test_no_columns_view_admin_errors_meaningfully(self):
        admin = NoColumnsExportAdmin(Manufacturer, 'testproject')
        request = self.factory.post('/')
        queryset = Manufacturer.objects.all()
        with self.assertRaises(ImproperlyConfigured):
            admin.export_csv_action(request, queryset)



from __future__ import unicode_literals

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from separated.views import CsvView


class CsvExportAdminMixin(object):
    """
    Adds an Export to CSV action for ModelAdmins.  You can specify which
    columns to display in the CSV with the csv_export_columns attribute.
    csv_export_columns corresponds to CsvView.columns.  For more control, you
    can override csv_export_view_class to get all of the flexibility that
    CsvView provides.
    """
    csv_export_columns = None
    csv_export_view_class = CsvView

    def get_csv_export_columns(self, request):
        return self.csv_export_columns

    def get_csv_export_view_class(self, request):
        return self.csv_export_view_class

    def export_csv_action(self, request, queryset):
        initkwargs = {
            'queryset': queryset,
        }

        columns = self.get_csv_export_columns(request)
        # maybe they already set the columns on the view.
        if columns is not None:
            initkwargs['columns'] = columns

        csv_view_class = self.get_csv_export_view_class(request)
        viewfn = csv_view_class.as_view(**initkwargs)
        # The base CsvView does not respond to POST requests, actions are only
        # ever POST requests.
        request.method = 'GET'
        return viewfn(request)
    export_csv_action.short_description = _("Export to CSV")


class CsvExportModelAdmin(CsvExportAdminMixin, admin.ModelAdmin):
    actions = ['export_csv_action']



import os
import sys

from setuptools import setup

version = __import__('separated').get_version()

install_requires = ['Django>=1.3']
tests_require = []

if sys.version_info[0] < 3:
    install_requires.append('unicodecsv')


def read_file(filename):
    with open(os.path.join(os.path.dirname(__file__), filename)) as f:
        return f.read()

setup(name='django-separated',
      version=version,
      author="Fusionbox, Inc.",
      author_email="programmers@fusionbox.com",
      url="https://github.com/fusionbox/django-separated",
      keywords="django csv class-based views",
      description="Class-based view and mixins for handling CSV with Django.",
      long_description=read_file('README.rst'),
      classifiers=[
          'Development Status :: 4 - Beta',
          'Framework :: Django',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Topic :: Internet :: WWW/HTTP',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
      ],
      install_requires=install_requires,
      tests_require=tests_require,
      packages=[
          'separated',
      ],

      test_suite='testproject.runtests',
)
